/*******************************************************************************
*                                 ZM4xx Demo
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      ametal.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief ZM4xx Demo
 *
 * - 裺
 *   1.ʹUSBDEMOԣ򿪴֣Ϊ115200λ8bitֹͣλ1bitУλ
 *   2.ʹôն˷ݰ߰°S2ݰ
 *   3.ն˵ĴֿԲ鿴յݣ
 *   4.S3Ըı䷢Ƶʡ
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */
 
#include "stdbool.h"
#include "stddef.h"
#include "LPC11xx.h"                                                    /* LPC11xxĴ            */
#include "uart.h"
#include "stdint.h"
#include "radio.h"
#include "int.h"
#include "stdio.h"
#include "timer.h"


#define FREQ          (433000000)

#define TEST_TX_PKTS  100

static uint8_t        __g_data_buf[50] = {0};
static uint32_t        __g_recv_pkt_cnt  = 0;                             
static uint32_t       __g_recv_byte     = 0;                                
static uint32_t        __g_send_byte     = 0;
static radio_handle_t __gp_handle;

extern void led_display (uint8_t val);

static void __delay_ms (uint32_t ms)
{
    uint32_t i = 0;
    uint32_t j = 0;

    while (i < ms) {
        i++;
        for(j = 0; j < 4369; j++);
    }
}

/**
 * \brief  S2ӦжϷһ
 */
static void __key2_int_handle (void)
{
    uint16_t  i = 0;
    uint32_t   len =  sizeof( __g_data_buf);
    __g_send_byte  = len;
    for (i = 0; i < len; i++) {
        __g_data_buf[i] = i;
    }
}

/**
 * \brief  S3жϷ,ı䷢书(5~20dBm)
 */
static void __key3_int_handle (void) 
{   
    static uint32_t freq    = 400000000;
    
    if (radio_freq_set(__gp_handle, freq) != 0) {
        printf("\r\n freq set error!!! \r\n");
    } else {
        radio_mode_set(__gp_handle, RX_MODE);
        printf("\r\n freq set:%d \r\n", freq);
    }
    freq += 1000000;
}

/**
 * \brief  ģжϷ
 */
static void __radio_recv_int_handle (void)
{
    int      ret = 0;
    uint8_t  len;
    
    ret = radio_buf_recv(__gp_handle, __g_data_buf, &len);  /*    */
    if (ret == RADIO_RET_OK) {
        __g_recv_byte = len;
    }
}

void demo_zm4xx_pkts_rate_entry (radio_handle_t handle)
{
    uint8_t  pkt_size = 0;
    uint16_t i = 0;
    uint32_t j = 0;
    uint32_t tx_us = 0;
    uint32_t err_pkts_cnt = 0;
    uint32_t rx_pkts_cnt = 0;
    uint32_t rx_bytes_cnt = 0;
    uint32_t rx_timout = 3000;
    uint32_t rx_us = 0;
    bool     is_rx_pkt = false;
    
    __gp_handle = handle;

      /* Ƶ */
    radio_freq_set(handle, FREQ);
    
    /* Ϊ״̬  */
    radio_mode_set(handle, RX_MODE);

    /* ʼģж  */
    radio_int_init(handle,__radio_recv_int_handle);
    
    key2_int_init(__key2_int_handle);
    key3_int_init(__key3_int_handle);
    
    led_display(0x18);

    while (1) {

        /*    */
        if (__g_recv_byte > 0) {
                    
            /* ǵһʼʱ */
            if (rx_pkts_cnt == 0) {
                timer0_32_start();
            }
            
            /* У */
            for (i = 0; i < __g_recv_byte; i++) {
                if (__g_data_buf[i] != i) {
                    err_pkts_cnt++;
                    break;
                }
            }
            
            is_rx_pkt = true;

            rx_pkts_cnt++;
            rx_bytes_cnt += __g_recv_byte;
            __g_recv_byte = 0;
            radio_mode_set(handle, RX_MODE); /* ½ģʽ */
            rx_timout = 3000;
            rx_us = timer0_32_tc_get();
        }
                

        /*    */
        if (__g_send_byte > 0) {            
            NVIC_DisableIRQ(UART_IRQn);       /* UARTж     */
            key_irq_disable();              /* رհж */
            
            led_display(0x80);
                    
            timer0_32_start();
            for (j = 0; j < TEST_TX_PKTS; j++) {
                radio_buf_send(handle, __g_data_buf, __g_send_byte); 
            }
            tx_us = timer0_32_stop();
            printf("tx data rate:%f \r\n", (double)(__g_send_byte * 8 * TEST_TX_PKTS) / ((double)tx_us / 1000000));
 
                        
            radio_mode_set(handle, RX_MODE);
            
            led_display(__g_recv_pkt_cnt % 128);
            __g_send_byte = 0;
            NVIC_EnableIRQ(UART_IRQn);        /* ʹUARTж */
            key_irq_enable();               /* ж */
        }

        /* ӴڻȡҪ͵ݰ */
        if (uart_recv_pkt(__g_data_buf, &pkt_size) == 0) {
            __g_send_byte = pkt_size;
        }

        if (rx_timout > 0) {
            rx_timout--;
            __delay_ms(1);
        } else {
            if (is_rx_pkt) { 
                printf("rx pkts:%d err pkts:%d data rate:%fbps\r\n",  \
                                         rx_pkts_cnt, err_pkts_cnt, (double)( rx_bytes_cnt * 8)/ ((double)rx_us / 1000000));
                is_rx_pkt    = false;
                rx_pkts_cnt  = 0;
                err_pkts_cnt = 0;
                rx_bytes_cnt = 0;
            }
        }                
    }
}

