/*******************************************************************************
*                                 UART Driver
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      ametal.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief UART
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */
 
#ifndef __UART_H
#define __UART_H

#define IRQ_FLAG_MASK           0x0000000E
#define RX_LINE_STATE           (0x03)
#define RX_DATA_VALID           (0x02)
#define CHAR_TIMEOUT_INDICATE   (0x06)

#define RX_DATA_READY           0x00000001

typedef void (*uart_rx_callback_t) (void *p_arg);

typedef struct uart { 
    uint32_t           baud_rate;
    uint16_t           charater_timeout;    /* ַʱ */
    uint16_t           buf_size;
    uint8_t           *p_buf;
    uint16_t           bytes_cnt;
    uint8_t            time_out;
    uart_rx_callback_t pfn_rx_callback;
} uart_t; 


void uart_init (uint32_t baud_rate, uint16_t buf_size, uart_rx_callback_t rx_callback);	
void uart_byte_send (uint8_t byte);
int  uart_recv_pkt (uint8_t *p_buf, uint8_t *p_size);
void uart_send_str (uint8_t const *pucStr, uint32_t ulNum);

void uart_en_rx_irq (void);
void uart_clear_rx_irq (void);

#endif 
