/*******************************************************************************
*                                 SPI Driver 
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2017 Guangzhou ZHIYUAN Electronics Stock Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      ametal.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief SPI
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */

#include "spi.h"
#include "LPC11xx.h"                                                    /* LPC11xxĴ            */
#include "LPC11xx_pinsel.h"
#include "LPC11xx_ssp.h"

#define TIME                 1        /* ڿIOģSPIʣݲͬMCUٶǰΪMCUƵΪ48MHz    */


#define MISO_PIN        (1ul << 8)                              /* ģSPIţ            */
#define CLK_PIN         (1ul << 6)                              /* SPIʱӣƵʲҪ5MHz    */
#define MOSI_PIN        (1ul << 9)                              /* ģSPI              */

#define MOSI_L()        LPC_GPIO0->DATA &= ~MOSI_PIN
#define MOSI_H()        LPC_GPIO0->DATA |=  MOSI_PIN

#define MISO_READ()     ((LPC_GPIO0->DATA & MISO_PIN) != 0)

#define CLK_L()         LPC_GPIO0->DATA &= ~CLK_PIN
#define CLK_H()         LPC_GPIO0->DATA |=  CLK_PIN

static void __delay_us (unsigned int i)
{
    unsigned int j, k;

    for (j = 0; j < 1; j++){
        for (k = 0; k < i; k++);
    }
}

#if HARDWARE_SPI

#define SBL_CLKPWR_PCONP_PCSSP                (1<<8)
#define SSP_CPHA_USED                         (SSP_CPHA_FIRST)
#define SSP_CPOL_USED                         (SSP_CPOL_HI)
#define SSP_DATA_BIT_USED                     (SSP_DATABIT_8)
#define SSP_DATA_WORD                         (0)
#define SSP_MODE_USED                         (SSP_MASTER_MODE)

void spi_send_byte (uint8_t send_data)
{
    if (LPC_SSP0->SR & SSP_SR_TNF) {
        LPC_SSP0->DR = send_data;
        while (LPC_SSP0->SR & SSP_SR_BSY);  /*  ȴ  */
    }
}


uint8_t spi_recv_byte (void)
{
    unsigned char tmp;

    while (LPC_SSP0->SR & SSP_SR_RNE) { /*  սյ  */
        tmp = LPC_SSP0->DR;
    }

    if (LPC_SSP0->SR & SSP_SR_TNF) {
        LPC_SSP0->DR = 0xFF;                /*  Ϳ  */
        while (LPC_SSP0->SR & SSP_SR_BSY);  /*  ȴ  */
    }

    if (LPC_SSP0->SR & SSP_SR_RNE) {
        return LPC_SSP0->DR;
    } else {
        return 0xFF;
    }
}


int spi_init (void)
{
    uint32_t tmp;

//    LPC_GPIO0->DIR |= SEL_C_PIN + SEL_D_PIN;    /*  ƬѡSEL_CSEL_DΪ   */
//    SEL_D_H();
//    SEL_C_H();

    // initialize pinsel for SSP0
    /* MISO0: P0.8 */
    LPC_IOCON->PIO0_8 = PINSEL_PIO_FUNC(1) | PINSEL_PIO_MODE(2);
    /* MOSI0: P0.9 */
    LPC_IOCON->PIO0_9 = PINSEL_PIO_FUNC(1) | PINSEL_PIO_MODE(2);

    /* P0.6 function 2 is SSP clock, need to combined with IOCONSCKLOC register setting */
    LPC_IOCON->SCK_LOC = 0x02;
    LPC_IOCON->PIO0_6 = PINSEL_PIO_FUNC(2) | PINSEL_PIO_MODE(2);

    /* Reset SSP0 peripheral */
    LPC_SYSCON->PRESETCTRL |= (0x1<<0);
    /* Enable clock for SSP0 */
    LPC_SYSCON->SYSAHBCLKCTRL |= (1<<11);
    LPC_SYSCON->SSP0CLKDIV = 0x02;


    /* Configure SSP, interrupt is disable, LoopBack mode is disable,
     * SSP is disable, Slave output is disable as default
     */
    tmp = ((SSP_CPHA_USED) | (SSP_CPOL_USED) \
            | (SSP_FRAME_SPI) | (SSP_DATA_BIT_USED))
            & SSP_CR0_BITMASK;
    // write back to SSP control register
    LPC_SSP0->CR0 = tmp;

    // Write back to CR1
    LPC_SSP0->CR1 = SSP_MASTER_MODE;

    // Set clock rate for SSP peripheral
    LPC_SSP0->CR0 &= (~ SSP_CR0_SCR (0xFF)) & SSP_CR0_BITMASK;
    LPC_SSP0->CR0 |= (SSP_CR0_SCR(0x05)) & SSP_CR0_BITMASK;
    LPC_SSP0->CPSR = 0x04 & SSP_CPSR_BITMASK;   /*  SPIʱΪ921600Hz  */

    // SSP Enable
    LPC_SSP0->CR1 |= SSP_CR1_SSP_EN;
}

#else

void spi_send_byte (uint8_t send_data)
{
    uint8_t i;

    for (i = 0; i < 8; i++) {
        __delay_us(TIME);
        CLK_L();                                                        /* clk=0;                       */
        if((send_data << i) & 0x80){
            __delay_us(TIME);
            MOSI_H();                                                    /* set 1;                       */
        } else {
            __delay_us(TIME);
            MOSI_L();                                                    /* set 0;                       */
        }

        __delay_us(TIME);
        CLK_H();                                                        /* clk=1;                       */
    }

    __delay_us(TIME);
    CLK_L();
    MOSI_L();                                                            /* set 0;                       */
}

uint8_t spi_recv_byte (void)
{
    uint8_t i, temp;

    temp = 0;
    for (i = 0; i < 8; i++) {
        CLK_L();                                                        /* clk=0;                       */
        __delay_us(TIME);
        temp = (temp << 1);
        CLK_H();                                                         /* clk=1;                       */
        __delay_us(TIME);
        if (MISO_READ()) {
            temp++;
        } else {                                                               /* set 1;                       */
            temp = temp + 0;
        }

        __delay_us(TIME);
    }

    CLK_L();
    __delay_us(TIME);

    return temp;
}

int spi_init (void)
{
    LPC_GPIO0->DIR    |= MOSI_PIN + CLK_PIN;  //
    LPC_GPIO0->DIR    &= ~(MISO_PIN);         //

    CLK_L();    //ų̬
    MOSI_L();   //ų̬
    
    return 0;
}

#endif
