/*******************************************************************************
*                                 UART Driver
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-present Guangzhou ZHIYUAN Electronics Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      ametal.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief UART
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */
#include "LPC11xx.h"                                                    /* LPC11xxĴ            */
#include "stdio.h"
#include "string.h"
#include "uart.h" 
#include "stdlib.h"
 
#ifdef __GNUC__
  /* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
     set to 'Yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */
  
/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  None
  * @retval None
  */
PUTCHAR_PROTOTYPE
{
  /* Place your implementation of fputc here */
  /* e.g. write a character to the USART */
  uart_byte_send((uint8_t)ch);

  return ch;
}

#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}

#endif

static uart_t    __g_uart_dev;
static uint8_t   __g_uart_init = 0;
extern uint16_t  g_time_out;

void uart_byte_send (uint8_t byte);

static void __delay_ms (uint32_t ms)
{
    uint32_t i = 0;
    uint32_t j = 0;

    while (i < ms) {
        i++;
        for(j = 0; j < 4369; j++);
    }
}



/* ڳʼ */
void uart_init (uint32_t baud_rate, uint16_t buf_size, uart_rx_callback_t rx_callback)
{
    uint16_t usFdiv;
    uint8_t  *p_buf;
    
    if (__g_uart_init) {
        return;
    }
    
    LPC_SYSCON->SYSAHBCLKCTRL |= (1 << 16);                             /* ʹIOCONʱ               */
    LPC_IOCON->PIO1_6 |= 0x01;                                          /* P1.6 1.7ΪRXDTXD    */
    LPC_IOCON->PIO1_7 |= 0x01;

    LPC_SYSCON->SYSAHBCLKCTRL |= (1<<12);                               /* UARTܲʱ         */
    LPC_SYSCON->UARTCLKDIV       = 0x01;                                /* UARTʱӷƵ                 */

    LPC_UART->LCR  = 0x83;                                              /* ò               */
    usFdiv = (SystemFrequency / LPC_SYSCON->UARTCLKDIV / 16) / baud_rate;      /* ò                   */
    LPC_UART->DLM  = usFdiv / 256;
    LPC_UART->DLL  = usFdiv % 256;
    LPC_UART->LCR  = 0x03;                                              /*                    */
    LPC_UART->FCR  = 0x87;                                              /* ʹFIFO8ֽڴ  */
    
    NVIC_EnableIRQ(UART_IRQn);                                          /* ʹUARTжϣȼ   */
    NVIC_SetPriority(UART_IRQn, 1);

    LPC_UART->IER  = 0x01;                                              /* ʹܽж                 */
   
    p_buf = (uint8_t *)malloc(buf_size);
    if (p_buf == NULL) {
        while(1);
    }
    
    __g_uart_dev.baud_rate       = baud_rate;
    __g_uart_dev.p_buf           = p_buf;
    __g_uart_dev.buf_size        = buf_size;
    __g_uart_dev.pfn_rx_callback = rx_callback;
    __g_uart_init = 1;                                                  /* ־ֹٴγʼ */
}



/* ʹܴж */
void uart_en_rx_irq (void)
{
    LPC_UART->IER  = 0x01;                                              /* ʹܽж                 */
}



/* رմж */
void uart_clear_rx_irq (void)
{
    LPC_UART->IER  = 0x00;                                              /* ʧܽж                 */
}



void uart_byte_send (uint8_t byte)
{
    LPC_UART->THR = byte;                                              /*  д                    */
    while ((LPC_UART->LSR & 0x40) == 0);                                /*  ȴݷ            */
}



void uart_send_str (uint8_t const *p_str, uint32_t len)
{
    uint32_t i;

    for (i = 0; i < len; i++){                                        /* ָֽ           */
        uart_byte_send (*p_str++);
    }
}




void UART_IRQHandler (void)
{
    while ((LPC_UART->IIR & 0x01) == 0) {                               /*  жǷжϹ          */
        __g_uart_dev.time_out = 150000 / __g_uart_dev.baud_rate + 2;    /*  趨ʱ봮й    */
		switch ((LPC_UART->IIR & IRQ_FLAG_MASK) >> 1) {                 /*  жжϱ־               */       
            case RX_DATA_VALID:                                         /*  ж               */
            case CHAR_TIMEOUT_INDICATE:                                 /*  ַʱж                */
                while (LPC_UART->LSR & RX_DATA_READY) {                 
                    if (__g_uart_dev.bytes_cnt >= __g_uart_dev.buf_size) {
                        break;
                    }
                  
                    __g_uart_dev.p_buf[__g_uart_dev.bytes_cnt] = LPC_UART->RBR;
                    __g_uart_dev.bytes_cnt++;
                }                
                break;
                
            default:
                break;
        }
    } 
}



int uart_recv_pkt (uint8_t *p_buf, uint8_t *p_size)
{
    int ret = -1;
    
    if (__g_uart_dev.bytes_cnt > 0) {
        if (__g_uart_dev.time_out > 0) {
            __delay_ms(1);
            __g_uart_dev.time_out--;
        } else {
            memcpy(p_buf, __g_uart_dev.p_buf, __g_uart_dev.bytes_cnt);
            *p_size = __g_uart_dev.bytes_cnt;
            __g_uart_dev.bytes_cnt = 0;
            ret = 0;
        }
    }
    
    return ret;
}