/*******************************************************************************
*                                  sx127x Driver
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2017 Guangzhou ZHIYUAN Electronics Stock Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      ametal.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief  sx127x Driver
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */
 
#ifndef __sx127x_H__
#define __sx127x_H__

#include "stdbool.h"
#include "radio.h"

#define XTAL_FREQ               32000000
#define FREQ_STEP               61.03515625
#define AM_LORA_RX_BUFFER_SIZE  256

#define CAD_STATE_START         0x01
#define CAD_STATE_FINISH        0x02
#define CAD_STATE_DONE          0x04
#define CAD_STATE_DETECT        0x08

#define CONTINUOUS_RECV_MODE    0
#define SINGLE_RECV_MODE        1

#define EXPLICIT_HEADER         0
#define IMPLICIT_HEADER         1

/**
 *  LoRa
 */
typedef struct sx127x_lora_param_set {
    uint32_t frequency;              /* Ƶ */
    int8_t   power;                  /* 书 */
    uint8_t  bandwidth;              /*  [0: 7.8 kHz, 1: 10.4 kHz, 2: 15.6 kHz, 3: 20.8 kHz, 4: 31.2 kHz,
                                              5: 41.6 kHz, 6: 62.5 kHz, 7: 125 kHz, 8: 250 kHz,  9: 500 kHz, other: Reserved] */
    uint8_t  sf;                     /* Ƶ [6: 64, 7: 128, 8: 256, 9: 512, 10: 1024, 11: 2048, 12: 4096  chips] */
    uint8_t  coderate;               /*  [1: 4/5, 2: 4/6, 3: 4/7, 4: 4/8] */
    bool     crc_on;                 /* CRC [0: ر, 1: ] */
    bool     header_mode;            /* ͷģʽ [0: ʽͷģʽ, 1: ʽͷģʽ] עSFΪ6ʱֻʹʽͷ */
    bool     rx_mode;                /* ģʽ [0: Continuous, 1 Single] */
    bool     freq_hop_on;            /* Ƶ [0: ر, 1: ] */
    uint8_t  hop_period;             /* Ƶڣsymbols */
    uint32_t tx_pkt_timeout;         /* ͳʱ */
    uint32_t rx_pkt_timeout;         /* ճʱ */
    uint8_t  payload_len;            /* س */
    uint8_t  preamble_len;           /* ǰ볤 */
    const uint32_t *p_hop_tab;       /* Ƶ */
    uint32_t  hop_tab_len;           /* Ƶ */
} sx127x_lora_param_set_t;

/**
 * RF packet definition
 */
#define RF_BUFFER_SIZE_MAX                          256
#define RF_BUFFER_SIZE                              256

/**
 * RF state machine
 */
typedef enum {
    RFLR_STATE_IDLE,
    RFLR_STATE_RX_INIT,
    RFLR_STATE_RX_RUNNING,
    RFLR_STATE_RX_DONE,
    RFLR_STATE_RX_TIMEOUT,
    RFLR_STATE_TX_INIT,
    RFLR_STATE_TX_RUNNING,
    RFLR_STATE_TX_DONE,
    RFLR_STATE_TX_TIMEOUT,
    RFLR_STATE_CAD_INIT,
    RFLR_STATE_CAD_RUNNING,
} sx127x_lora_sm_t;



/**
 * RegFormerTemp
 */
typedef struct __sx127x_lora_reg {
    uint8_t RegFifo;                                // 0x00
    // Common settings
    uint8_t RegOpMode;                              // 0x01
    uint8_t RegRes02;                               // 0x02
    uint8_t RegRes03;                               // 0x03
    uint8_t RegBandSetting;                         // 0x04
    uint8_t RegRes05;                               // 0x05
    uint8_t RegFrfMsb;                              // 0x06
    uint8_t RegFrfMid;                              // 0x07
    uint8_t RegFrfLsb;                              // 0x08
    // Tx settings
    uint8_t RegPaConfig;                            // 0x09
    uint8_t RegPaRamp;                              // 0x0A
    uint8_t RegOcp;                                 // 0x0B
    // Rx settings
    uint8_t RegLna;                                 // 0x0C
    // LoRa registers
    uint8_t RegFifoAddrPtr;                         // 0x0D
    uint8_t RegFifoTxBaseAddr;                      // 0x0E
    uint8_t RegFifoRxBaseAddr;                      // 0x0F
    uint8_t RegFifoRxCurrentAddr;                   // 0x10
    uint8_t RegIrqFlagsMask;                        // 0x11
    uint8_t RegIrqFlags;                            // 0x12
    uint8_t RegNbRxBytes;                           // 0x13
    uint8_t RegRxHeaderCntValueMsb;                 // 0x14
    uint8_t RegRxHeaderCntValueLsb;                 // 0x15
    uint8_t RegRxPacketCntValueMsb;                 // 0x16
    uint8_t RegRxPacketCntValueLsb;                 // 0x17
    uint8_t RegModemStat;                           // 0x18
    uint8_t RegPktSnrValue;                         // 0x19
    uint8_t RegPktRssiValue;                        // 0x1A
    uint8_t RegRssiValue;                           // 0x1B
    uint8_t RegHopChannel;                          // 0x1C
    uint8_t RegModemConfig1;                        // 0x1D
    uint8_t RegModemConfig2;                        // 0x1E
    uint8_t RegSymbTimeoutLsb;                      // 0x1F
    uint8_t RegPreambleMsb;                         // 0x20
    uint8_t RegPreambleLsb;                         // 0x21
    uint8_t RegPayloadLength;                       // 0x22
    uint8_t RegMaxPayloadLength;                    // 0x23
    uint8_t RegHopPeriod;                           // 0x24
    uint8_t RegFifoRxByteAddr;                      // 0x25
    uint8_t RegModemConfig3;                        // 0x26
    uint8_t RegTestReserved27[0x30 - 0x27];         // 0x27-0x30
    uint8_t RegTestReserved31;                      // 0x31
    uint8_t RegTestReserved32[0x40 - 0x32];         // 0x32-0x40
    // I/O settings
    uint8_t RegDioMapping1;                         // 0x40
    uint8_t RegDioMapping2;                         // 0x41
    // Version
    uint8_t RegVersion;                             // 0x42
    // Additional settings
    uint8_t RegAgcRef;                              // 0x43
    uint8_t RegAgcThresh1;                          // 0x44
    uint8_t RegAgcThresh2;                          // 0x45
    uint8_t RegAgcThresh3;                          // 0x46
    // Test
    uint8_t RegTestReserved47[0x4B - 0x47];         // 0x47-0x4A
    // Additional settings
    uint8_t RegPllHop;                              // 0x4B
    uint8_t RegTestReserved4C;                      // 0x4C
    uint8_t RegPaDac;                               // 0x4D
    // Test
    uint8_t RegTestReserved4E[0x58-0x4E];           // 0x4E-0x57
    // Additional settings
    uint8_t RegTcxo;                                // 0x58
    // Test
    uint8_t RegTestReserved59;                      // 0x59
    // Test
    uint8_t RegTestReserved5B;                      // 0x5B
    // Additional settings
    uint8_t RegPll;                                 // 0x5C
    // Test
    uint8_t RegTestReserved5D;                      // 0x5D
    // Additional settings
    uint8_t RegPllLowPn;                            // 0x5E
    // Test
    uint8_t RegTestReserved5F[0x6C - 0x5F];         // 0x5F-0x6B
    // Additional settings
    uint8_t RegFormerTemp;                          // 0x6C
    // Test
    uint8_t RegTestReserved6D[0x71 - 0x6D];         // 0x6D-0x70
} __sx127x_lora_reg_t;



enum {
	LORA_RET_CODE_OK = 0,
	LORA_RET_CODE_ERR,
	LORA_RET_CODE_NO_CAD_DONE_IRQ,
	LORA_RET_CODE_NO_CAD_DETECT_IRQ,
	LORA_RET_CAD_RUNNING,
	LORA_RET_RECV_CRC_ERR
};



typedef struct spi_dev {
    void    (*pfn_spi_write_byte) (uint8_t byte);
    uint8_t (*pfn_spi_read_byte)  (void);
} sx127x_spi_funcs_t;




typedef struct radio_sx127x_gpio {
    void    (*pfn_reset_pin_set) (uint8_t level);
    void    (*pfn_sel_pin_set)   (uint8_t level);
    uint8_t (*pfn_dio0_pin_read) (void);
} sx127x_gpio_funcs_t;




typedef struct radio_sx127x_timer {
    void (*pfn_delay_ms) (uint32_t ms);
    void (*pfn_delay_us) (uint32_t us);
} sx127x_timer_funcs_t;




typedef struct sx127x_pkt_state {
    bool    crc_pass;
    bool    pkt_rx_done;
    bool    pkt_tx_done;
    int8_t  snr;
    int16_t rssi;
    uint8_t pkt_buffer[AM_LORA_RX_BUFFER_SIZE];
    uint8_t pkt_len;
} sx127x_pkt_info_t;



typedef struct {

    /**
     * \brief ɻصԭ
     */
    void (*pfn_tx_done_cb)(void);

    /**
     * \brief 䳬ʱصԭ
     */
    void (*pfn_tx_timeout_cb)(void);

    /**
     * \brief ɻصԭ
     *
     * \param[in] p_payload : ָջ
     * \param[in] size      : յݴС
     * \param[in] rssi      : ո֡ʱ RSSI ֵ [dBm]
     * \param[in] snr       : Ӳԭʼ
     *                         FSK : N/A (set to 0)
     *                         LoRa: SNR value in dB
     */
    void (*pfn_rx_done_cb)(uint8_t *p_payload, uint16_t size, int16_t rssi, int8_t snr);

    /**
     * \brief ճʱصԭ
     */
    void (*pfn_rx_timeout_cb)(void);

    /**
     * \brief մصԭ
     */
    void (*pfn_rx_error_cb)(void);

    /**
     * \brief FHSSƵصԭ
     *
     * \param[in] current_channel : ǰŵı
     */
    void (*pfn_fhss_change_channel_cb)(uint8_t current_channel);

    /**
     * \brief ŵԾɻصԭ
     *
     * \param[in] channel_activity_detected : ǷԾ
     */
    void (*pfn_cad_done_cb)(bool channel_activity_detected);

} sx127x_lora_events_t;



typedef struct sx127x_setting {
    uint8_t          cad_state;
    sx127x_lora_sm_t sm_state;
} sx127x_run_state_t;



typedef struct sx127x_config {
	bool                     rx_pkt_crc_err;
    sx127x_lora_events_t    *p_events;
    sx127x_lora_param_set_t *p_lora_param;
} sx127x_config_t;



typedef struct radio_sx127x_dev {
    radio_dev_t                isa;
    sx127x_spi_funcs_t        *p_spi;
    sx127x_gpio_funcs_t       *p_gpio;
    sx127x_timer_funcs_t      *p_delay;
    sx127x_run_state_t         run_state;
    sx127x_pkt_info_t          pkt_info;
    //sx127x_setting_t           setting;
    //radio_state_t              run_state;
    const sx127x_config_t     *p_config;
} radio_sx127x_dev_t;




void __sx127x_on_dio0_irq (radio_sx127x_dev_t *p_this);

void __sx127x_on_dio1_irq (radio_sx127x_dev_t *p_this);

void __sx127x_on_dio2_irq (radio_sx127x_dev_t *p_this);

void __sx127x_on_dio3_irq (radio_sx127x_dev_t *p_this);

void __sx127x_on_dio5_irq (radio_sx127x_dev_t *p_this);


/**
 * \brief ʼ
 *
 * \param[in]  p_sx127x  : 豸
 * \param[in]  p_spi     : SPI
 * \param[in]  p_gpio    : GPIO
 * \param[in]  p_delay   : ʱ
 * \param[in]  p_info    : 豸Ϣ
 *
 * \retval  NULL        : ʼʧ
 * \retval  ֵ       ʼɹ
 */
radio_handle_t radio_sx127x_init (radio_sx127x_dev_t   *p_sx127x,
                                  sx127x_spi_funcs_t   *p_spi,
                                  sx127x_gpio_funcs_t  *p_gpio,
                                  sx127x_timer_funcs_t *p_delay,
                                  sx127x_config_t      *p_config);


#endif //__sx127x_H__



