/*******************************************************************************
*                            SX127x Radio Config
*                       ----------------------------
*                       innovating embedded platform
*
* Copyright (c) 2001-2017 Guangzhou ZHIYUAN Electronics Stock Co., Ltd.
* All rights reserved.
*
* Contact information:
* web site:    http://www.zlg.cn/
* e-mail:      ametal.support@zlg.cn
*******************************************************************************/

/**
 * \file
 * \brief SX127xģû
 *     ûֲʱֻҪṩSPIGPIOʱɡ
 *
 * \internal
 * \par Modification history
 * - 1.00 17-11-22  sup, first implementation.
 * \endinternal
 */
 
#include "stdio.h"
#include "sx127x_lora.h"
#include "zm4xx_gpio.h"
#include "timer.h"
#include "spi.h"

static sx127x_spi_funcs_t      __g_spi_dev;
static sx127x_gpio_funcs_t     __g_gpio_dev;
static sx127x_timer_funcs_t	   __g_delay;
static radio_sx127x_dev_t      __g_sx127x;
static sx127x_config_t         __g_sx127x_cfg;
static sx127x_lora_events_t    __g_sx127x_events;



/*
 * Ƶ
 */
static const uint32_t __g_hop_freq_tab[64] =
{
    420000000,
    421000000,
    422000000,
    423000000,
    424000000,
    425000000,
    426000000,
    427000000,
    428000000,
    429000000,
    430000000,
    431000000,
    432000000,
    433000000,
    434000000,
    435000000,
    436000000,
    437000000,
    438000000,
    439000000,
    440000000,
    441000000,
    442000000,
    443000000,
    444000000,
    445000000,
    446000000,
    447000000,
    448000000,
    449000000,
    450000000,
    451000000,
    452000000,
    453000000,
    454000000,
    455000000,
    456000000,
    457000000,
    458000000,
    459000000,
    460000000,
    461000000,
    462000000,
    463000000,
    464000000,
    465000000,
    466000000,
    467000000,
    468000000,
    469000000,
    470000000,
    471000000,
    472000000,
    473000000,
    474000000,
    475000000,
    476000000,
    477000000,
    478000000,
    479000000,
    480000000,
    481000000,
    482000000,
    483000000
};



/*
 * Lora
 */
static sx127x_lora_param_set_t __g_lora_param =
{
    472000000,       /* Ƶ */
    20,              /* 书 */
    8,               /*  [0: 7.8 kHz, 1: 10.4 kHz, 2: 15.6 kHz, 3: 20.8 kHz, 4: 31.2 kHz,
                              5: 41.6 kHz, 6: 62.5 kHz, 7: 125 kHz, 8: 250 kHz,  9: 500 kHz, other: Reserved] */
    7,               /* Ƶ [6: 64, 7: 128, 8: 256, 9: 512, 10: 1024, 11: 2048, 12: 4096  chips] */
    1,               /*    [1: 4/5, 2: 4/6, 3: 4/7, 4: 4/8] */
    1,               /* CRC  [0: ر, 1: ] */
    0,               /* ͷģʽ [0: ʽͷģʽ, 1: ʽͷģʽ] עSFΪ6ʱֻʹʽͷ */
    0,               /* ģʽ [0: Continuous, 1 Single] */
    0,               /* Ƶ     [0: ر, 1: ] */
    4,               /* Ƶڣsymbols */
    100,             /* ͳʱ */
    100,             /* ճʱ */
    10,              /* س */
    8,               /* ǰ볤 */
    __g_hop_freq_tab,/* Ƶ */
    64               /* Ƶ */
};



static void __cad_done_call_func (bool ch_active)
{
    //printf("cad done!\r\n");
}    



static void __rx_timeout_call_func (bool ch_active)
{
    //printf("rx timeout!\r\n");
}



static void __rx_crc_err_call_func (void)
{
    //printf("rx crc err!\r\n");
}
    
    

static void __fhss_change_channel_call_func (uint8_t current_channel)
{
    //printf("fssh cur channel:%d\r\n", current_channel);
}



static void __rx_done_call_func (uint8_t *p_payload, uint16_t size, int16_t rssi, int8_t snr)
{
    //printf("pkt rssi:%d   snr:%d \r\n", rssi, snr);
}
    
    
    
radio_handle_t radio_zm4xx_inst_init (void)
{    
    __g_spi_dev.pfn_spi_read_byte  = spi_recv_byte;
    __g_spi_dev.pfn_spi_write_byte = spi_send_byte;

    __g_gpio_dev.pfn_reset_pin_set = zm4xx_rst_pin_set; 
    __g_gpio_dev.pfn_sel_pin_set   = zm4xx_sel_pin_set;
    __g_gpio_dev.pfn_dio0_pin_read = zm4xx_dio0_pin_read;
    
    __g_delay.pfn_delay_ms = timer0_16_delay_ms;
    __g_delay.pfn_delay_us = timer0_16_delay_us;
    
    __g_sx127x_events.pfn_rx_done_cb             = __rx_done_call_func;
    __g_sx127x_events.pfn_cad_done_cb            = __cad_done_call_func;
    __g_sx127x_events.pfn_rx_timeout_cb          = __rx_timeout_call_func;
    __g_sx127x_events.pfn_rx_error_cb            = __rx_crc_err_call_func;
    __g_sx127x_events.pfn_fhss_change_channel_cb = __fhss_change_channel_call_func;

    __g_sx127x_cfg.p_events     = &__g_sx127x_events;
    __g_sx127x_cfg.p_lora_param = &__g_lora_param;

    return radio_sx127x_init(&__g_sx127x, 
                             &__g_spi_dev,
                             &__g_gpio_dev,
                             &__g_delay,
                             &__g_sx127x_cfg);
}


